@@ -1,7 +1,7 @@
 """Rotations in three dimensions - SO(3)."""
 import warnings
+import math
 import numpy as np
-from .plot_utils import Frame, Arrow3D, make_3d_axis
 from numpy.testing import assert_array_almost_equal
 
 
@@ -48,12 +48,12 @@ def norm_angle(a):
     Parameters
     ----------
     a : float or array-like, shape (n,)
-        Angle(s)
+        Angle(s) in radians
 
     Returns
     -------
     a_norm : float or array-like, shape (n,)
-        Normalized angle(s)
+        Normalized angle(s) in radians
     """
     # Source of the solution: http://stackoverflow.com/a/32266181
     return -((np.pi - np.asarray(a)) % (2.0 * np.pi) - np.pi)
@@ -82,10 +82,7 @@ def norm_axis_angle(a):
     res = np.empty(4)
     res[:3] = a[:3] / norm
 
-    while angle < 0.0:
-        angle += 2.0 * np.pi
-    while angle > np.pi:
-        angle -= 2.0 * np.pi
+    angle = norm_angle(angle)
     if angle < 0.0:
         angle *= -1.0
         res[:3] *= -1.0
@@ -95,6 +92,27 @@ def norm_axis_angle(a):
     return res
 
 
+def norm_compact_axis_angle(a):
+    """Normalize compact axis-angle representation.
+
+    Parameters
+    ----------
+    a : array-like, shape (3,)
+        Axis of rotation and rotation angle: angle * (x, y, z)
+
+    Returns
+    -------
+    a : array-like, shape (3,)
+        Axis of rotation and rotation angle: angle * (x, y, z).
+        The angle is in [0, pi). No rotation is represented by [0, 0, 0].
+    """
+    angle = np.linalg.norm(a)
+    if angle == 0.0:
+        return np.zeros(3)
+    axis = a / angle
+    return axis * norm_angle(angle)
+
+
 def perpendicular_to_vectors(a, b):
     """Compute perpendicular vector to two other vectors.
 
@@ -114,6 +132,33 @@ def perpendicular_to_vectors(a, b):
     return np.cross(a, b)
 
 
+def perpendicular_to_vector(a):
+    """Compute perpendicular vector to one other vector.
+
+    There is an infinite number of solutions to this problem. Thus, we
+    restrict the solutions to [1, 0, z] and return [0, 0, 1] if the
+    z component of a is 0.
+
+    Parameters
+    ----------
+    a : array-like, shape (3,)
+        3d vector
+
+    Returns
+    -------
+    b : array-like, shape (3,)
+        A 3d vector that is orthogonal to a. It does not necessarily have
+        unit length.
+    """
+    if abs(a[2]) < eps:
+        return np.copy(unitz)
+    # Now that we solved the problem for [x, y, 0], we can solve it for all
+    # other vectors by restricting solutions to [1, 0, z] and find z.
+    # The dot product of orthogonal vectors is 0, thus
+    # a[0] * 1 + a[1] * 0 + a[2] * z == 0 or -a[0] / a[2] = z
+    return np.array([1.0, 0.0, -a[0] / a[2]])
+
+
 def angle_between_vectors(a, b, fast=False):
     """Compute angle between two vectors.
 
@@ -186,6 +231,28 @@ def random_axis_angle(random_state=np.random.RandomState(0)):
     return a
 
 
+def random_compact_axis_angle(random_state=np.random.RandomState(0)):
+    """Generate random compact axis-angle.
+
+    The angle will be sampled uniformly from the interval :math:`[0, \pi)`
+    and each component of the rotation axis will be sampled from
+    :math:`\mathcal{N}(\mu=0, \sigma=1)` and than the axis will be normalized
+    to length 1.
+
+    Parameters
+    ----------
+    random_state : np.random.RandomState, optional (default: random seed 0)
+        Random number generator
+
+    Returns
+    -------
+    a : array-like, shape (3,)
+        Axis of rotation and rotation angle: angle * (x, y, z)
+    """
+    a = random_axis_angle(random_state)
+    return a[:3] * a[3]
+
+
 def random_quaternion(random_state=np.random.RandomState(0)):
     """Generate random quaternion.
 
@@ -300,6 +367,26 @@ def check_axis_angle(a):
     return norm_axis_angle(a)
 
 
+def check_compact_axis_angle(a):
+    """Input validation of compact axis-angle representation.
+
+    Parameters
+    ----------
+    a : array-like, shape (3,)
+        Axis of rotation and rotation angle: angle * (x, y, z)
+
+    Returns
+    -------
+    a : array, shape (3,)
+        Validated axis of rotation and rotation angle: angle * (x, y, z)
+    """
+    a = np.asarray(a, dtype=np.float)
+    if a.ndim != 1 or a.shape[0] != 3:
+        raise ValueError("Expected axis and angle in array with shape (3,), "
+                         "got array-like object with shape %s" % (a.shape,))
+    return norm_compact_axis_angle(a)
+
+
 def check_quaternion(q, unit=True):
     """Input validation of quaternion representation.
 
@@ -326,15 +413,38 @@ def check_quaternion(q, unit=True):
         return q
 
 
+def check_quaternions(Q, unit=True):
+    """Input validation of quaternion representation.
+
+    Parameters
+    ----------
+    Q : array-like, shape (n_steps, 4)
+        Quaternions to represent rotations: (w, x, y, z)
+
+    unit : bool, optional (default: True)
+        Normalize the quaternions so that they are unit quaternions
+
+    Returns
+    -------
+    Q : array-like, shape (n_steps, 4)
+        Validated quaternions to represent rotations: (w, x, y, z)
+    """
+    Q_checked = np.asarray(Q, dtype=np.float)
+    if Q_checked.ndim != 2 or Q_checked.shape[1] != 4:
+        raise ValueError("Expected quaternion array with shape (n_steps, 4), got "
+                         "array-like object with shape %s" % (Q_checked.shape,))
+    if unit:
+        for i in range(len(Q)):
+            Q_checked[i] = norm_vector(Q_checked[i])
+    return Q_checked
+
+
 def matrix_from_axis_angle(a):
     """Compute rotation matrix from axis-angle.
 
     This is called exponential map or Rodrigues' formula.
 
-    Note that the rotation convention is different from the one used by Euler
-    angles. A rotation about a basis vector results in a transposed rotation
-    matrix in comparison to a corresponding Euler angles. The active / alibi
-    convention is used.
+    This typically results in an active rotation matrix.
 
     Parameters
     ----------
@@ -348,8 +458,8 @@ def matrix_from_axis_angle(a):
     """
     a = check_axis_angle(a)
     ux, uy, uz, theta = a
-    c = np.cos(theta)
-    s = np.sin(theta)
+    c = math.cos(theta)
+    s = math.sin(theta)
     ci = 1.0 - c
     R = np.array([[ci * ux * ux + c,
                    ci * ux * uy - uz * s,
@@ -363,16 +473,39 @@ def matrix_from_axis_angle(a):
                   ])
 
     # This is equivalent to
-    # R = (np.eye(3) * np.cos(theta) +
-    #      (1.0 - np.cos(theta)) * a[:3, np.newaxis].dot(a[np.newaxis, :3]) +
-    #      cross_product_matrix(a[:3]) * np.sin(theta))
+    # R = (np.eye(3) * np.cos(a[3]) +
+    #      (1.0 - np.cos(a[3])) * a[:3, np.newaxis].dot(a[np.newaxis, :3]) +
+    #      cross_product_matrix(a[:3]) * np.sin(a[3]))
 
     return R
 
 
+def matrix_from_compact_axis_angle(a):
+    """Compute rotation matrix from compact axis-angle.
+
+    This is called exponential map or Rodrigues' formula.
+
+    This typically results in an active rotation matrix.
+
+    Parameters
+    ----------
+    a : array-like, shape (3,)
+        Axis of rotation and rotation angle: angle * (x, y, z)
+
+    Returns
+    -------
+    R : array-like, shape (3, 3)
+        Rotation matrix
+    """
+    a = axis_angle_from_compact_axis_angle(a)
+    return matrix_from_axis_angle(a)
+
+
 def matrix_from_quaternion(q):
     """Compute rotation matrix from quaternion.
 
+    This typically results in an active rotation matrix.
+
     Parameters
     ----------
     q : array-like, shape (4,)
@@ -403,13 +536,7 @@ def matrix_from_quaternion(q):
 
 
 def matrix_from_angle(basis, angle):
-    """Compute rotation matrix from rotation around basis vector.
-
-    The combined rotation matrices are either extrinsic and can be used with
-    pre-multiplied column vectors or they are intrinsic and can be used with
-    post-multiplied row vectors. We use a right-hand system with right-hand
-    rotations. We use the passive / alias convention. You can derive the
-    active / alibi rotation matrix by transposing the rotation matrix.
+    """Compute passive rotation matrix from rotation around basis vector.
 
     Parameters
     ----------
@@ -445,12 +572,48 @@ def matrix_from_angle(basis, angle):
     return R
 
 
-def matrix_from_euler_xyz(e):
-    """Compute rotation matrix from xyz Euler angles.
+passive_matrix_from_angle = matrix_from_angle
 
-    Intrinsic rotations are used to create the transformation matrix
-    from three concatenated rotations.
-    The xyz convention is usually used in physics and chemistry.
+
+def active_matrix_from_angle(basis, angle):
+    """Compute active rotation matrix from rotation around basis vector.
+
+    Parameters
+    ----------
+    basis : int from [0, 1, 2]
+        The rotation axis (0: x, 1: y, 2: z)
+
+    angle : float
+        Rotation angle
+
+    Returns
+    -------
+    R : array-like, shape (3, 3)
+        Rotation matrix
+    """
+    c = np.cos(angle)
+    s = np.sin(angle)
+
+    if basis == 0:
+        R = np.array([[1.0, 0.0, 0.0],
+                      [0.0, c, -s],
+                      [0.0, s, c]])
+    elif basis == 1:
+        R = np.array([[c, 0.0, s],
+                      [0.0, 1.0, 0.0],
+                      [-s, 0.0, c]])
+    elif basis == 2:
+        R = np.array([[c, -s, 0.0],
+                      [s, c, 0.0],
+                      [0.0, 0.0, 1.0]])
+    else:
+        raise ValueError("Basis must be in [0, 1, 2]")
+
+    return R
+
+
+def matrix_from_euler_xyz(e):
+    """Compute passive rotation matrix from intrinsic xyz Tait-Bryan angles.
 
     Parameters
     ----------
@@ -463,20 +626,14 @@ def matrix_from_euler_xyz(e):
         Rotation matrix
     """
     alpha, beta, gamma = e
-    # We use intrinsic rotations
-    Qx = matrix_from_angle(0, alpha)
-    Qy = matrix_from_angle(1, beta)
-    Qz = matrix_from_angle(2, gamma)
-    R = Qx.dot(Qy).dot(Qz)
+    R = passive_matrix_from_angle(0, alpha).dot(
+        passive_matrix_from_angle(1, beta)).dot(
+        passive_matrix_from_angle(2, gamma))
     return R
 
 
 def matrix_from_euler_zyx(e):
-    """Compute rotation matrix from zyx (yaw-pitch-roll) Euler angles.
-
-    Intrinsic rotations are used to create the transformation matrix
-    from three concatenated rotations.
-    The zyx convention is usually used for aircraft dynamics.
+    """Compute passive rotation matrix from intrinsic zyx Tait-Bryan angles.
 
     Parameters
     ----------
@@ -489,11 +646,105 @@ def matrix_from_euler_zyx(e):
         Rotation matrix
     """
     gamma, beta, alpha = e
-    # We use intrinsic rotations
-    Qz = matrix_from_angle(2, gamma)
-    Qy = matrix_from_angle(1, beta)
-    Qx = matrix_from_angle(0, alpha)
-    R = Qz.dot(Qy).dot(Qx)
+    R = passive_matrix_from_angle(2, gamma).dot(
+        passive_matrix_from_angle(1, beta)).dot(
+        passive_matrix_from_angle(0, alpha))
+    return R
+
+
+def active_matrix_from_intrinsic_euler_zxz(e):
+    """Compute active rotation matrix from intrinsic zxz Euler angles.
+
+    Parameters
+    ----------
+    e : array-like, shape (3,)
+        Angles for rotation around z-, x'-, and z''-axes (intrinsic rotations)
+
+    Returns
+    -------
+    R : array-like, shape (3, 3)
+        Rotation matrix
+    """
+    alpha, beta, gamma = e
+    R = active_matrix_from_angle(2, alpha).dot(
+        active_matrix_from_angle(0, beta)).dot(
+        active_matrix_from_angle(2, gamma))
+    return R
+
+
+def active_matrix_from_extrinsic_euler_zxz(e):
+    """Compute active rotation matrix from extrinsic zxz Euler angles.
+
+    Parameters
+    ----------
+    e : array-like, shape (3,)
+        Angles for rotation around z-, x-, and z-axes (extrinsic rotations)
+
+    Returns
+    -------
+    R : array-like, shape (3, 3)
+        Rotation matrix
+    """
+    # The difference between extrinsic and intrinsic rotation is the order of
+    # rotations and because the reverse of zxz is zxz this is exactly the same.
+    return active_matrix_from_intrinsic_euler_zxz(e)
+
+
+def active_matrix_from_intrinsic_euler_zyz(e):
+    """Compute active rotation matrix from intrinsic zyz Euler angles.
+
+    Parameters
+    ----------
+    e : array-like, shape (3,)
+        Angles for rotation around z-, y'-, and z''-axes (intrinsic rotations)
+
+    Returns
+    -------
+    R : array-like, shape (3, 3)
+        Rotation matrix
+    """
+    alpha, beta, gamma = e
+    R = active_matrix_from_angle(2, alpha).dot(
+        active_matrix_from_angle(1, beta)).dot(
+        active_matrix_from_angle(2, gamma))
+    return R
+
+
+def active_matrix_from_extrinsic_euler_zyz(e):
+    """Compute active rotation matrix from extrinsic zyz Euler angles.
+
+    Parameters
+    ----------
+    e : array-like, shape (3,)
+        Angles for rotation around z-, y-, and z-axes (extrinsic rotations)
+
+    Returns
+    -------
+    R : array-like, shape (3, 3)
+        Rotation matrix
+    """
+    # The difference between extrinsic and intrinsic rotation is the order of
+    # rotations and because the reverse of zyz is zyz this is exactly the same.
+    return active_matrix_from_intrinsic_euler_zyz(e)
+
+
+def active_matrix_from_extrinsic_roll_pitch_yaw(rpy):
+    """Compute active rotation matrix from extrinsic roll, pitch, and yaw.
+
+    Parameters
+    ----------
+    rpy : array-like, shape (3,)
+        Angles for rotation around x- (roll), y- (pitch), and z-axes (yaw), extrinsic rotations
+
+    Returns
+    -------
+    R : array-like, shape (3, 3)
+        Rotation matrix
+    """
+    roll, pitch, yaw = rpy
+    R = active_matrix_from_angle(2, yaw).dot(
+        active_matrix_from_angle(1, pitch)).dot(
+        active_matrix_from_angle(0, roll))
     return R
 
 
@@ -536,12 +787,12 @@ def matrix_from(R=None, a=None, q=None, e_xyz=None, e_zyx=None):
 
 
 def euler_xyz_from_matrix(R, strict_check=True):
-    """Compute xyz Euler angles from rotation matrix.
+    """Compute xyz Euler angles from passive rotation matrix.
 
     Parameters
     ----------
     R : array-like, shape (3, 3)
-        Rotation matrix
+        Passive rotation matrix
 
     strict_check : bool, optional (default: True)
         Raise a ValueError if the rotation matrix is not numerically close
@@ -571,12 +822,12 @@ def euler_xyz_from_matrix(R, strict_check=True):
 
 
 def euler_zyx_from_matrix(R, strict_check=True):
-    """Compute zyx Euler angles from rotation matrix.
+    """Compute zyx Euler angles from passive rotation matrix.
 
     Parameters
     ----------
     R : array-like, shape (3, 3)
-        Rotation matrix
+        Passive rotation matrix
 
     strict_check : bool, optional (default: True)
         Raise a ValueError if the rotation matrix is not numerically close
@@ -613,10 +864,7 @@ def axis_angle_from_matrix(R, strict_check=True):
     This operation is called logarithmic map. Note that there are two possible
     solutions for the rotation axis when the angle is 180 degrees (pi).
 
-    Note that the rotation convention is different from the one used by Euler
-    angles. A rotation about a basis vector results in a transposed rotation
-    matrix in comparison to a corresponding Euler angles. The active / alibi
-    convention is used.
+    We usually assume active rotations.
 
     Parameters
     ----------
@@ -674,6 +922,8 @@ def axis_angle_from_quaternion(q):
 
     This operation is called logarithmic map.
 
+    We usually assume active rotations.
+
     Parameters
     ----------
     q : array-like, shape (4,)
@@ -696,6 +946,67 @@ def axis_angle_from_quaternion(q):
         return np.hstack((axis, angle))
 
 
+def axis_angle_from_compact_axis_angle(a):
+    """Compute axis-angle from compact axis-angle representation.
+
+    We usually assume active rotations.
+
+    Parameters
+    ----------
+    a : array-like, shape (3,)
+        Axis of rotation and rotation angle: angle * (x, y, z).
+
+    Returns
+    -------
+    a : array-like, shape (4,)
+        Axis of rotation and rotation angle: (x, y, z, angle). The angle is
+        constrained to [0, pi].
+    """
+    a = check_compact_axis_angle(a)
+    angle = np.linalg.norm(a)
+    if angle == 0.0:
+        return np.array([1.0, 0.0, 0.0, 0.0])
+    else:
+        axis = a / angle
+        return np.hstack((axis, (angle,)))
+
+
+def axis_angle_from_two_directions(a, b):
+    """Compute axis-angle representation from two direction vectors.
+
+    The rotation will transform direction vector a to direction vector b.
+    The direction vectors don't have to be normalized as this will be
+    done internally. Note that there is more than one possible solution.
+
+    Parameters
+    ----------
+    a : array-like, shape (3,)
+        First direction vector
+
+    b : array-like, shape (3,)
+        Second direction vector
+
+    Returns
+    -------
+    a : array-like, shape (4,)
+        Axis of rotation and rotation angle: (x, y, z, angle). The angle is
+        constrained to [0, pi].
+    """
+    a = norm_vector(a)
+    b = norm_vector(b)
+    cos_angle = a.dot(b)
+    if abs(-1.0 - cos_angle) < eps:
+        # For 180 degree rotations we have an infinite number of solutions,
+        # but we have to pick one axis.
+        axis = perpendicular_to_vector(a)
+    else:
+        axis = np.cross(a, b)
+    aa = np.empty(4)
+    aa[:3] = norm_vector(axis)
+    aa[3] = np.arccos(cos_angle)
+    return norm_axis_angle(aa)
+
+
 def compact_axis_angle(a):
     """Compute 3-dimensional axis-angle from a 4-dimensional one.
 
@@ -704,6 +1015,8 @@ def compact_axis_angle(a):
     transform :math:`\\left( \\boldsymbol{\hat{e}}, \\theta \\right)` to
     :math:`\\theta \\boldsymbol{\hat{e}}`.
 
+    We usually assume active rotations.
+
     Parameters
     ----------
     a : array-like, shape (4,)
@@ -712,18 +1025,65 @@ def compact_axis_angle(a):
     Returns
     -------
     a : array-like, shape (3,)
-        Compact representation of axis of rotation and rotation angle. a is
-        the rotation axis and np.linalg.norm(a) is the rotation angle.
+        Axis of rotation and rotation angle: angle * (x, y, z) (compact
+        representation).
     """
-    angle = a[3]
-    if angle == 0.0:
-        return np.zeros(3)
-    return a[:3] / np.linalg.norm(a[:3]) * angle
+    a = check_axis_angle(a)
+    return a[:3] * a[3]
+
+
+def compact_axis_angle_from_matrix(R):
+    """Compute compact axis-angle from rotation matrix.
+
+    This operation is called logarithmic map. Note that there are two possible
+    solutions for the rotation axis when the angle is 180 degrees (pi).
+
+    We usually assume active rotations.
+
+    Parameters
+    ----------
+    R : array-like, shape (3, 3)
+        Rotation matrix
+
+    strict_check : bool, optional (default: True)
+        Raise a ValueError if the rotation matrix is not numerically close
+        enough to a real rotation matrix. Otherwise we print a warning.
+
+    Returns
+    -------
+    a : array-like, shape (3,)
+        Axis of rotation and rotation angle: angle * (x, y, z). The angle is
+        constrained to [0, pi].
+    """
+    a = axis_angle_from_matrix(R)
+    return compact_axis_angle(a)
+
+
+def compact_axis_angle_from_quaternion(q):
+    """Compute compact axis-angle from quaternion (logarithmic map).
+
+    We usually assume active rotations.
+
+    Parameters
+    ----------
+    q : array-like, shape (4,)
+        Unit quaternion to represent rotation: (w, x, y, z)
+
+    Returns
+    -------
+    a : array-like, shape (3,)
+        Axis of rotation and rotation angle: angle * (x, y, z). The angle is
+        constrained to [0, pi].
+    """
+    a = axis_angle_from_quaternion(q)
+    return compact_axis_angle(a)
 
 
 def quaternion_from_matrix(R, strict_check=True):
     """Compute quaternion from rotation matrix.
 
+    We usually assume active rotations.
+
     .. warning::
 
         When computing a quaternion from the rotation matrix there is a sign
@@ -781,6 +1141,8 @@ def quaternion_from_axis_angle(a):
 
     This operation is called exponential map.
 
+    We usually assume active rotations.
+
     Parameters
     ----------
     a : array-like, shape (4,)
@@ -800,6 +1162,25 @@ def quaternion_from_axis_angle(a):
     return q
 
 
+def quaternion_from_compact_axis_angle(a):
+    """Compute quaternion from compact axis-angle (exponential map).
+
+    We usually assume active rotations.
+
+    Parameters
+    ----------
+    a : array-like, shape (4,)
+        Axis of rotation and rotation angle: angle * (x, y, z)
+
+    Returns
+    -------
+    q : array-like, shape (4,)
+        Unit quaternion to represent rotation: (w, x, y, z)
+    """
+    a = axis_angle_from_compact_axis_angle(a)
+    return quaternion_from_axis_angle(a)
+
+
 def quaternion_xyzw_from_wxyz(q_wxyz):
     """Converts from w, x, y, z to x, y, z, w convention.
 
@@ -834,6 +1215,72 @@ def quaternion_wxyz_from_xyzw(q_xyzw):
     return np.array([q_xyzw[3], q_xyzw[0], q_xyzw[1], q_xyzw[2]])
 
 
+def quaternion_integrate(Qd, q0=np.array([1.0, 0.0, 0.0, 0.0]), dt=1.0):
+    """Integrate angular velocities to quaternions.
+
+    Parameters
+    ----------
+    A : array-like, shape (n_steps, 3)
+        Angular velocities in a compact axis-angle representation. Each angular
+        velocity represents the rotational offset after one unit of time.
+
+    q0 : array-like, shape (4,), optional (default: [1, 0, 0, 0])
+        Unit quaternion to represent initial rotation: (w, x, y, z)
+
+    dt : float, optional (default: 1)
+        Time interval between steps.
+
+    Returns
+    -------
+    Q : array-like, shape (n_steps, 4)
+        Quaternions to represent rotations: (w, x, y, z)
+    """
+    Q = np.empty((len(Qd), 4))
+    Q[0] = q0
+    for t in range(1, len(Qd)):
+        qd = (Qd[t] + Qd[t - 1]) / 2.0
+        Q[t] = concatenate_quaternions(
+            quaternion_from_compact_axis_angle(dt * qd), Q[t - 1])
+    return Q
+
+
+def quaternion_gradient(Q, dt=1.0):
+    """Time-derivatives of a sequence of quaternions.
+
+    Note that this function does not provide the exact same functionality for
+    quaternions as [NumPy's gradient
+    function](https://numpy.org/doc/stable/reference/generated/numpy.gradient.html)
+    for positions. Gradients are always computed as central differences except
+    the first and last gradient. We additionally accept a parameter dt that
+    defines the time interval between each quaternion. Note that this means
+    that we expect this to be constant for the whole sequence.
+
+    Parameters
+    ----------
+    Q : array-like, shape (n_steps, 4)
+        Quaternions to represent rotations: (w, x, y, z)
+
+    dt : float, optional (default: 1)
+        Time interval between steps. If you have non-constant dt, you can pass
+        1 and manually divide angular velocities by their corresponding time
+        interval afterwards.
+
+    Returns
+    -------
+    A : array-like, shape (n_steps, 3)
+        Angular velocities in a compact axis-angle representation. Each angular
+        velocity represents the rotational offset after one unit of time.
+    """
+    Q = check_quaternions(Q)
+    Qd = np.empty((len(Q), 3))
+    Qd[0] = compact_axis_angle_from_quaternion(concatenate_quaternions(Q[1], q_conj(Q[0]))) / dt
+    for t in range(1, len(Q) - 1):
+        # divided by two because of central differences
+        Qd[t] = compact_axis_angle_from_quaternion(concatenate_quaternions(Q[t + 1], q_conj(Q[t - 1]))) / (2.0 * dt)
+    Qd[-1] = compact_axis_angle_from_quaternion(concatenate_quaternions(Q[-1], q_conj(Q[-2]))) / dt
+    return Qd
+
+
 def concatenate_quaternions(q1, q2):
     """Concatenate two quaternions.
 
@@ -1054,6 +1501,7 @@ def plot_basis(ax=None, R=None, p=np.zeros(3), s=1.0, ax_s=1, strict_check=True,
     ax : Matplotlib 3d axis
         New or old axis
     """
+    from .plot_utils import make_3d_axis, Frame
     if ax is None:
         ax = make_3d_axis(ax_s)
 
@@ -1099,6 +1547,7 @@ def plot_axis_angle(ax=None, a=a_id, p=p0, s=1.0, ax_s=1, **kwargs):
     ax : Matplotlib 3d axis
         New or old axis
     """
+    from .plot_utils import make_3d_axis, Arrow3D
     a = check_axis_angle(a)
     if ax is None:
         ax = make_3d_axis(ax_s)
@@ -1142,6 +1591,14 @@ def assert_axis_angle_equal(a1, a2, *args, **kwargs):
     constraints and will normalize the representations before comparison.
     See numpy.testing.assert_array_almost_equal for a more detailed
     documentation of the other parameters.
+
+    Parameters
+    ----------
+    a1 : array-like, shape (4,)
+        Axis of rotation and rotation angle: (x, y, z, angle)
+
+    a2 : array-like, shape (4,)
+        Axis of rotation and rotation angle: (x, y, z, angle)
     """
     # required despite normalization in case of 180 degree rotation
     if np.any(np.sign(a1) != np.sign(a2)):
@@ -1151,6 +1608,33 @@ def assert_axis_angle_equal(a1, a2, *args, **kwargs):
     assert_array_almost_equal(a1, a2, *args, **kwargs)
 
 
+def assert_compact_axis_angle_equal(a1, a2, *args, **kwargs):
+    """Raise an assertion if two axis-angle are not approximately equal.
+
+    Usually we assume that the angle is within [0, pi). However, this function
+    ignores this constraint and will normalize the representations before
+    comparison. See numpy.testing.assert_array_almost_equal for a more detailed
+    documentation of the other parameters.
+
+    Parameters
+    ----------
+    a1 : array-like, shape (3,)
+        Axis of rotation and rotation angle: angle * (x, y, z)
+
+    a2 : array-like, shape (3,)
+        Axis of rotation and rotation angle: angle * (x, y, z)
+    """
+    angle1 = np.linalg.norm(a1)
+    angle2 = np.linalg.norm(a2)
+    # required despite normalization in case of 180 degree rotation
+    if (abs(angle1) == np.pi and abs(angle2) == np.pi and
+        any(np.sign(a1) != np.sign(a2))):
+        a1 = -a1
+    a1 = norm_compact_axis_angle(a1)
+    a2 = norm_compact_axis_angle(a2)
+    assert_array_almost_equal(a1, a2, *args, **kwargs)
+
+
 def assert_quaternion_equal(q1, q2, *args, **kwargs):
     """Raise an assertion if two quaternions are not approximately equal.
 
