@@ -1,13 +1,11 @@
 """Load transformations from URDF files."""
 import os
-import warnings
 import numpy as np
-from mpl_toolkits import mplot3d
 from bs4 import BeautifulSoup
 from .transform_manager import TransformManager
-from .transformations import transform_from, concat, transform
-from .rotations import matrix_from_euler_xyz, matrix_from_axis_angle
-from .plot_utils import make_3d_axis
+from .transformations import transform_from, concat
+from .rotations import active_matrix_from_extrinsic_roll_pitch_yaw, matrix_from_axis_angle, norm_vector
+from .plot_utils import make_3d_axis, plot_mesh, plot_cylinder, plot_sphere, plot_box
 
 
 class UrdfTransformManager(TransformManager):
@@ -25,15 +23,26 @@ class UrdfTransformManager(TransformManager):
     .. note::
 
         Joint angles must be given in radians.
+
+    Parameters
+    ----------
+    strict_check : bool, optional (default: True)
+        Raise a ValueError if the transformation matrix is not numerically
+        close enough to a real transformation matrix. Otherwise we print a
+        warning.
+
+    check : bool, optional (default: True)
+        Check if transformation matrices are valid and requested nodes exist,
+        which might significantly slow down some operations.
     """
-    def __init__(self):
-        super(UrdfTransformManager, self).__init__()
+    def __init__(self, strict_check=True, check=True):
+        super(UrdfTransformManager, self).__init__(strict_check, check)
         self._joints = {}
         self.collision_objects = []
         self.visuals = []
 
     def add_joint(self, joint_name, from_frame, to_frame, child2parent, axis,
-                  limits=(float("-inf"), float("inf"))):
+                  limits=(float("-inf"), float("inf")), joint_type="revolute"):
         """Add joint.
 
         Parameters
@@ -55,31 +64,47 @@ class UrdfTransformManager(TransformManager):
 
         limits : pair of float, optional (default: (-inf, inf))
             Lower and upper joint angle limit
+
+        joint_type : str, optional (default: 'revolute')
+            Joint type: revolute or prismatic (continuous is the same as
+            revolute)
         """
         self.add_transform(from_frame, to_frame, child2parent)
-        self._joints[joint_name] = (from_frame, to_frame, child2parent, axis,
-                                    limits)
+        self._joints[joint_name] = (from_frame, to_frame, child2parent, norm_vector(axis),
+                                    limits, joint_type)
 
-    def set_joint(self, joint_name, angle):
-        """Set joint angle.
+    def set_joint(self, joint_name, value):
+        """Set joint position.
 
-        Note that joint angles are clipped to their limits.
+        Note that joint values are clipped to their limits.
 
         Parameters
         ----------
         joint_name : string
             Name of the joint
 
-        angle : float
-            Joint angle in radians
+        value : float
+            Joint angle in radians in case of revolute joints or position
+            in case of prismatic joint.
         """
         if joint_name not in self._joints:
             raise KeyError("Joint '%s' is not known" % joint_name)
-        from_frame, to_frame, child2parent, axis, limits = self._joints[joint_name]
-        angle = np.clip(angle, limits[0], limits[1])
-        joint_rotation = matrix_from_axis_angle(np.hstack((axis, [angle])))
-        joint2A = transform_from(joint_rotation, np.zeros(3))
-        self.add_transform(from_frame, to_frame, concat(joint2A, child2parent))
+        from_frame, to_frame, child2parent, axis, limits, joint_type = self._joints[joint_name]
+        # this is way faster than np.clip:
+        value = min(max(value, limits[0]), limits[1])
+        if joint_type == "revolute":
+            joint_rotation = matrix_from_axis_angle(
+                np.hstack((axis, (value,))))
+            joint2A = transform_from(
+                joint_rotation, np.zeros(3), strict_check=self.strict_check)
+        else:
+            assert joint_type == "prismatic"
+            joint_offset = value * axis
+            joint2A = transform_from(
+                np.eye(3), joint_offset, strict_check=self.strict_check)
+        self.add_transform(from_frame, to_frame, concat(
+            joint2A, child2parent, strict_check=self.strict_check,
+            check=self.check))
 
     def get_joint_limits(self, joint_name):
         """Get limits of a joint.
@@ -125,31 +150,62 @@ class UrdfTransformManager(TransformManager):
 
         robot_name = robot["name"]
 
-        links = [self._parse_link(link)
+        materials = dict([
+            self._parse_material(material)
+            for material in robot.findAll("material", recursive=False)])
+
+        links = [self._parse_link(link, materials)
                  for link in robot.findAll("link", recursive=False)]
         joints = [self._parse_joint(joint, links)
                   for joint in robot.findAll("joint", recursive=False)]
 
         self.add_transform(links[0], robot_name, np.eye(4))
         for joint in joints:
-            if joint.joint_type == "revolute":
+            if joint.joint_type in ["revolute", "continuous"]:
                 self.add_joint(
                     joint.joint_name, joint.child, joint.parent,
-                    joint.child2parent, joint.joint_axis, joint.limits)
+                    joint.child2parent, joint.joint_axis, joint.limits,
+                    "revolute")
+            elif joint.joint_type == "prismatic":
+                self.add_joint(
+                    joint.joint_name, joint.child, joint.parent,
+                    joint.child2parent, joint.joint_axis, joint.limits,
+                    "prismatic")
             else:
+                assert joint.joint_type == "fixed"
                 self.add_transform(
                     joint.child, joint.parent, joint.child2parent)
 
-    def _parse_link(self, link):
+    def _parse_material(self, material):
+        """Parse material."""
+        if not material.has_attr("name"):
+            raise UrdfException("Material name is missing.")
+        colors = material.findAll("color")
+        if len(colors) not in [0, 1]:
+            raise UrdfException("More than one color is not allowed.")
+        if len(colors) == 1:
+            color = self._parse_color(colors[0])
+        else:
+            color = None
+        # TODO texture is currently ignored
+        return material["name"], color
+
+    def _parse_color(self, color):
+        """Parse color."""
+        if not color.has_attr("rgba"):
+            raise UrdfException("Attribute 'rgba' of color tag is missing.")
+        return np.fromstring(color["rgba"], sep=" ")
+
+    def _parse_link(self, link, materials):
         """Create link."""
         if not link.has_attr("name"):
             raise UrdfException("Link name is missing.")
-        self.visuals.extend(self._parse_link_children(link, "visual"))
+        self.visuals.extend(self._parse_link_children(link, "visual", materials))
         self.collision_objects.extend(
-            self._parse_link_children(link, "collision"))
+            self._parse_link_children(link, "collision", dict()))
         return link["name"]
 
-    def _parse_link_children(self, link, child_type):
+    def _parse_link_children(self, link, child_type, materials):
         """Parse collision objects or visuals."""
         children = link.findAll(child_type)
         shape_objects = []
@@ -158,12 +214,22 @@ class UrdfTransformManager(TransformManager):
                 name = child["name"]
             else:
                 name = "%s/%s_%s" % (link["name"], child_type, i)
+
+            color = None
+            if child_type == "visual":
+                material = child.find("material")
+                if material is not None:
+                    material_name, color = self._parse_material(material)
+                    if color is None and material_name in materials:
+                        color = materials[material_name]
+
             child2link = self._parse_origin(child)
             self.add_transform(name, link["name"], child2link)
-            shape_objects.extend(self._parse_geometry(child, name))
+
+            shape_objects.extend(self._parse_geometry(child, name, color))
         return shape_objects
 
-    def _parse_geometry(self, child, name):
+    def _parse_geometry(self, child, name, color):
         """Parse geometric primitives (box, cylinder, sphere) or meshes."""
         geometry = child.find("geometry")
         if geometry is None:
@@ -173,7 +239,7 @@ class UrdfTransformManager(TransformManager):
             shapes = geometry.findAll(shape_type)
             Cls = shape_classes[shape_type]
             for shape in shapes:
-                shape_object = Cls(name, mesh_path=self.mesh_path)
+                shape_object = Cls(name, mesh_path=self.mesh_path, color=color)
                 shape_object.parse(shape)
                 result.append(shape_object)
         return result
@@ -216,16 +282,16 @@ class UrdfTransformManager(TransformManager):
 
         j.joint_type = joint["type"]
 
-        if j.joint_type in ["planar", "floating", "continuous", "prismatic"]:
+        if j.joint_type in ["planar", "floating"]:
             raise UrdfException("Unsupported joint type '%s'" % j.joint_type)
-        elif j.joint_type not in ["revolute", "fixed"]:
+        elif j.joint_type not in ["revolute", "continuous", "prismatic", "fixed"]:
             raise UrdfException("Joint type '%s' is not allowed in a URDF "
                                 "document." % j.joint_type)
 
         j.child2parent = self._parse_origin(joint)
 
         j.joint_axis = np.array([1, 0, 0])
-        if j.joint_type == "revolute":
+        if j.joint_type in ["revolute", "continuous", "prismatic"]:
             axis = joint.find("axis")
             if axis is not None and axis.has_attr("xyz"):
                 j.joint_axis = np.fromstring(axis["xyz"], sep=" ")
@@ -244,13 +310,13 @@ class UrdfTransformManager(TransformManager):
             if origin.has_attr("rpy"):
                 roll_pitch_yaw = np.fromstring(origin["rpy"], sep=" ")
                 # URDF and KDL use the active convention for rotation matrices.
-                # To convert the defined rotation to the passive convention we
-                # must invert (transpose) the matrix. For more details on how
-                # the URDF parser handles the conversion from Euler angles,
-                # see this blog post:
+                # For more details on how the URDF parser handles the conversion
+                # from Euler angles, see this blog post:
                 # https://orbitalstation.wordpress.com/tag/quaternion/
-                rotation = matrix_from_euler_xyz(roll_pitch_yaw).T
-        return transform_from(rotation, translation)
+                rotation = active_matrix_from_extrinsic_roll_pitch_yaw(
+                    roll_pitch_yaw)
+        return transform_from(
+            rotation, translation, strict_check=self.strict_check)
 
     def _parse_limits(self, joint):
         """Parse joint limits."""
@@ -388,68 +454,41 @@ class Joint(object):
 
 
 class Box(object):
-    def __init__(self, frame, mesh_path):
+    def __init__(self, frame, mesh_path, color):
         self.frame = frame
+        self.color = color
 
     def parse(self, box):
         self.size = np.zeros(3)
         if box.has_attr("size"):
             self.size = np.fromstring(box["size"], sep=" ")
 
-    def plot(self, tm, frame, ax=None, color="k", wireframe=True):
+    def plot(self, tm, frame, ax=None, wireframe=True):
         A2B = tm.get_transform(self.frame, frame)
-
-        corners = np.array([
-            [0, 0, 0],
-            [0, 0, 1],
-            [0, 1, 0],
-            [0, 1, 1],
-            [1, 0, 0],
-            [1, 0, 1],
-            [1, 1, 0],
-            [1, 1, 1]
-        ])
-        corners = (corners - 0.5) * self.size
-        corners = transform(
-            A2B, np.hstack((corners, np.ones((len(corners), 1)))))[:, :3]
-
-        for i, j in [(0, 1), (0, 2), (1, 3), (2, 3),
-                     (4, 5), (4, 6), (5, 7), (6, 7),
-                     (0, 4), (1, 5), (2, 6), (3, 7)]:
-            ax.plot([corners[i, 0], corners[j, 0]],
-                    [corners[i, 1], corners[j, 1]],
-                    [corners[i, 2], corners[j, 2]], c=color)
-
-        return ax
+        color = self.color if self.color is not None else "k"
+        return plot_box(ax, self.size, A2B, wireframe=wireframe, color=color)
 
 
 class Sphere(object):
-    def __init__(self, frame, mesh_path):
+    def __init__(self, frame, mesh_path, color):
         self.frame = frame
+        self.color = color
 
     def parse(self, sphere):
         if not sphere.has_attr("radius"):
             raise UrdfException("Sphere has no radius.")
         self.radius = float(sphere["radius"])
 
-    def plot(self, tm, frame, ax=None, color="k", wireframe=True):
+    def plot(self, tm, frame, ax=None, wireframe=True):
         center = tm.get_transform(self.frame, frame)[:3, 3]
-        phi, theta = np.mgrid[0.0:np.pi:100j, 0.0:2.0 * np.pi:100j]
-        x = center[0] + self.radius * np.sin(phi) * np.cos(theta)
-        y = center[1] + self.radius * np.sin(phi) * np.sin(theta)
-        z = center[2] + self.radius * np.cos(phi)
-
-        if wireframe:
-            ax.plot_wireframe(x, y, z, rstride=10, cstride=10, color=color)
-        else:
-            ax.plot_surface(x, y, z, color=color, alpha=0.2, linewidth=0)
-
-        return ax
+        color = self.color if self.color is not None else "k"
+        return plot_sphere(ax, self.radius, center, wireframe=wireframe, color=color)
 
 
 class Cylinder(object):
-    def __init__(self, frame, mesh_path):
+    def __init__(self, frame, mesh_path, color):
         self.frame = frame
+        self.color = color
 
     def parse(self, cylinder):
         if not cylinder.has_attr("radius"):
@@ -459,45 +498,22 @@ class Cylinder(object):
             raise UrdfException("Cylinder has no length.")
         self.length = float(cylinder["length"])
 
-    def plot(self, tm, frame, ax=None, color="k", wireframe=True):
+    def plot(self, tm, frame, ax=None, wireframe=True):
         A2B = tm.get_transform(self.frame, frame)
-
-        axis_start = A2B.dot(np.array([0, 0, -0.5 * self.length, 1]))[:3]
-        axis_end = A2B.dot(np.array([0, 0, 0.5 * self.length, 1]))[:3]
-        axis = axis_end - axis_start
-        axis /= self.length
-
-        not_axis = np.array([1, 0, 0])
-        if (axis == not_axis).all():
-            not_axis = np.array([0, 1, 0])
-
-        n1 = np.cross(axis, not_axis)
-        n1 /= np.linalg.norm(n1)
-        n2 = np.cross(axis, n1)
-
-        t = np.linspace(0, self.length, 100)
-        theta = np.linspace(0, 2 * np.pi, 100)
-        t, theta = np.meshgrid(t, theta)
-        X, Y, Z = [axis_start[i] + axis[i] * t +
-                   self.radius * np.sin(theta) * n1[i] +
-                   self.radius * np.cos(theta) * n2[i] for i in [0, 1, 2]]
-
-        if wireframe:
-            ax.plot_wireframe(X, Y, Z, rstride=10, cstride=10, color=color)
-        else:
-            ax.plot_surface(X, Y, Z, color=color, alpha=0.2, linewidth=0)
-
-        return ax
+        color = self.color if self.color is not None else "k"
+        return plot_cylinder(ax, self.length, self.radius, 0.0, A2B, wireframe, color)
 
 
 class Mesh(object):
-    def __init__(self, frame, mesh_path):
+    def __init__(self, frame, mesh_path, color):
         self.frame = frame
         self.mesh_path = mesh_path
+        self.color = color
 
     def parse(self, mesh):
         if self.mesh_path is None:
             self.filename = None
+            self.scale = 1.0
         else:
             if not mesh.has_attr("filename"):
                 raise UrdfException("Mesh has no filename.")
@@ -508,35 +524,10 @@ class Mesh(object):
             else:
                 self.scale = np.ones(3)
 
-    def plot(self, tm, frame, ax=None, alpha=0.3):
-        if self.filename is None:
-            warnings.warn(
-                "Mesh will be ignored. You have to set a mesh path to "
-                "plot meshes.")
-            return ax
-
+    def plot(self, tm, frame, ax=None, alpha=0.3, convex_hull=True):
         A2B = tm.get_transform(self.frame, frame)
-
-        try:
-            import trimesh
-        except ImportError:
-            warnings.warn(
-                "Cannot display mesh. Library 'trimesh' not installed.")
-            return ax
-
-        mesh = trimesh.load(self.filename)
-        vertices = mesh.vertices * self.scale
-        vertices = np.hstack((vertices, np.ones((len(vertices), 1))))
-        vertices = transform(A2B, vertices)[:, :3]
-        vectors = np.array([vertices[[i, j, k]] for i, j, k in mesh.faces])
-
-        p3c = mplot3d.art3d.Poly3DCollection(vectors)
-        p3c.set_alpha(alpha)
-        # HACK without this line the alpha value would not work
-        p3c.set_facecolor(None)
-        ax.add_collection3d(p3c)
-
-        return ax
+        color = self.color if self.color is not None else "k"
+        return plot_mesh(ax, self.filename, A2B, self.scale, convex_hull=convex_hull, alpha=alpha, color=color)
 
 
 shape_classes = {"box": Box,
